/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1998-1999 Jeremie Miller.
 *
 * Portions Copyright (c) 2002 Matthias Wimmer.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 * --------------------------------------------------------------------------*/
 
#include "smtp.h"

/*

This is the smtp server component.

It acts as a simple SMTP server for both receiving email directly from another server or for delivering
email to another server.

What inspired me to write this very frightening thing is a mystery :) 

Jer

*/

/* takes foo@mapped.host or foo%jabber.org@smtp.host and translates to foo@jabber.org */

jid smtp_mx2(smtpi i, jid id)
{
    char *at, *trhost;
    xmlnode tr;

    if(id == NULL || id->user == NULL) return NULL;

    log_debug(ZONE,"mx2 checking %s",jid_full(id));

    // query for any translation
    tr = xmlnode_get_tag(i->config, spools(id->p, "map?mxhost=",id->server,id->p));
    trhost = xmlnode_get_attrib(tr,"jabber");

    if (trhost != NULL)
       jid_set(id,trhost,JID_SERVER);
    else
       {
       if ((at = strstr(id->user,"%")) != NULL)
          {
          *at = '@';
          id = jid_new(id->p,id->user);

          // check to make sure this id is valid to relay between 
          if (id != NULL && i->open == 0 && 
              xmlnode_get_tag(i->config,spools(id->p,"open=",id->server,id->p)) == NULL
             )
             return NULL;
          }
       else
          return NULL;
       }

    log_debug(ZONE,"mx2 returning %s",jid_full(id));
    return id;
}

/* takes foo@jabber.org and translates to foo@mapped.host or foo%jabber.org@smtp.host */
jid smtp_2mx(smtpi i, jid from)
{
    char *at, *trhost, *strid;
    jid id;
    xmlnode tr;

    if(from == NULL || from->user == NULL) return NULL;

    log_debug(ZONE,"2mx checking %s",jid_full(from));

    /* copy jid and modify copy for the result */
    id = jid_new(from->p,jid_full(from));
    jid_set(id,NULL,JID_RESOURCE);

    /* query for any translation */
    tr = xmlnode_get_tag(i->config, spools(id->p, "map?jabber=",id->server,id->p));
    trhost = xmlnode_get_attrib(tr,"mxhost");

    if (trhost != NULL)
       jid_set(id,trhost,JID_SERVER);
    else
       {
       strid = jid_full(id);

       at = strstr(strid,"@");
       *at = '%';
       id = jid_new(id->p,i->me);
       jid_set(id,strid,JID_USER);
       }

    log_debug(ZONE,"2mx returning %s",jid_full(id));
    return id;
}

void smtp(instance i, xmlnode x)
   {
   xmlnode config, cur, jep0033;
   xdbcache xc;
   smtpi si;
   char *lcPom;

   log_debug(ZONE,"smtp transport loading");

   xc = xdb_cache(i);
   config = xdb_get(xc, jid_new(xmlnode_pool(x),"config@-internal"),"jabber:config:smtp");


   /* set up internal instance and hash tables */
   si = pmalloco(i->p,sizeof(_smtpi));
   si->i = i;
   si->xc = xc;
   si->config = xmlnode_dup_pool(i->p, config);
   si->timeout = j_atoi(xmlnode_get_tag_data(config,"timeout"),30);
   si->JabberMessagesElapsedOut=si->JabberMessagesElapsedOutError=
	   si->JabberMessagesElapsed=si->SMTPMessagesElapsed=
	   si->SMTPMessagesElapsedIn=si->SMTPMessagesElapsedInError=
       si->SMTPElapsedSpam=si->SMTPElapsedCommandUnknown=
       si->jep0033=si->attachments=si->CoreContacts= si->CoreContactsOnline=0;

   si->cSMTP_Status[0]=si->cJabber_Status[0]=' ';
   si->cSMTP_Status[1]=si->cJabber_Status[1]='\0';

   // try create backup folder
   
     si->tmpBackupJabber=si->tmpBackupSMTP=0;

//   liBackup=mkdir("./spool/smtp",S_IRUSR | S_IWUSR);
//   if ( liBackup==0 || errno == EEXIST )
//      {
      if ((lcPom=xmlnode_get_tag_data(config,"backup?xmlns=smtp:backup:jabber"))==NULL)
         lcPom= SMTP_BACKUPJABBER_FOLDER;

      if (smtp_CreateFolder(lcPom)==0)
         si->tmpBackupJabber=1;

      if ((lcPom=xmlnode_get_tag_data(config,"backup?xmlns=smtp:backup:smtp"))==NULL)
         lcPom= SMTP_BACKUPSMTP_FOLDER;

      if (smtp_CreateFolder(lcPom)==0)
         si->tmpBackupSMTP=1;
//	  }
//   else
//      si->tmpBackupJabber=si->tmpBackupSMTP=0;

   // get default language
   if ((lcPom=xmlnode_get_tag_data(config,"language"))==NULL)
      lcPom= SMTP_RC_DEFAULT_LANG;

   si->DefaultLang=pstrdup(i->p, lcPom);

   // get statistics file name and read it
   if ((si->stat_file=xmlnode_get_tag_data(config,"statistics"))==NULL)
      si->stat_file=pstrdup(i->p, SMTP_STATISTICS_FILE);

   if ((si->statistics=xmlnode_file(si->stat_file))==NULL)
      { // file not exist or any other bug
      si->statistics=xmlnode_new_tag("smtp");
      xmlnode_put_attrib(si->statistics,"xmlns","jabber:status:smtp");

      xmlnode_put_attrib(si->statistics,"Start","0");
      xmlnode_put_attrib(si->statistics,"StartLastDatetime","0");

      xmlnode_put_attrib(si->statistics,"CoreContacts","0");
      xmlnode_put_attrib(si->statistics,"CoreContactsOnline","0");

	  
	  xmlnode_put_attrib(si->statistics,"JabberMessages","0");
      xmlnode_put_attrib(si->statistics,"JabberMessagesOut","0");
      xmlnode_put_attrib(si->statistics,"JabberMessagesOutError","0");

      xmlnode_put_attrib(si->statistics,"SMTPCommandUnknown","0");
      xmlnode_put_attrib(si->statistics,"SMTPMessages","0");
      xmlnode_put_attrib(si->statistics,"SMTPMessagesIn","0");
      xmlnode_put_attrib(si->statistics,"SMTPMessagesInError","0");

      xmlnode_put_attrib(si->statistics,"SMTPSpam","0");
      }  

   // get resources file name and read it
   if ((si->res_file=xmlnode_get_tag_data(config,"resources"))==NULL)
      si->res_file=pstrdup(i->p,SMTP_RESOURCE_FILE);

   if ((si->resources=xmlnode_file(si->res_file))==NULL)
      { // file not exist or any other bug
      log_error("smtp","SMTP Server must have resource file");
      return;
      }

   if ((si->me = xmlnode_get_tag_data(si->config,"mxhost")) == NULL)
      {
      log_error("smtp","SMTP Server must have a default SMTP <mxhost>smtp.host.foo</mxhost> configured");
      return;
      }

   for (cur = xmlnode_get_tag(config,"open");cur != NULL; xmlnode_hide(cur), cur = xmlnode_get_tag(config,"open"))
       if(xmlnode_get_data(cur) == NULL)
         si->open = 1;

   // attachments
   if ((cur = xmlnode_get_tag(si->config,"attachments"))!=NULL)
      si->attachments=j_atoi(xmlnode_get_attrib(cur, "pass"), 0);

   // relay smtp server
   cur = xmlnode_get_tag(si->config,"relay");
   si->relayport=j_atoi(xmlnode_get_attrib(cur, "port"), SMTP_Relay_Port);

   if((si->relay = xmlnode_get_attrib(cur,"ip")) == NULL)
       si->relay = pstrdup(i->p, SMTP_DEFAULT_RELAY);



   register_phandler(i, o_DELIVER, smtp_out_packets, (void *)si);


   si->spamcheck = xmlnode_get_tag_data(si->config,"spamcheck");
   // vytvoit vlastn thread
   if ((cur = xmlnode_get_tag(config,"ip")) != NULL)
      for (;cur != NULL; xmlnode_hide(cur), cur = xmlnode_get_tag(config,"ip"))
          {
          mio_listen(j_atoi(xmlnode_get_attrib(cur, "port"), SMTP_Incomming_Port), xmlnode_get_data(cur), smtp_in_read, (void*)si, NULL, NULL);
          }
   else // no special config, use defaults //
      mio_listen(SMTP_Incomming_Port, NULL, smtp_in_read, (void *)si, NULL, NULL);
 

   xmlnode_free(config);

   // JEP-0033 test
   cur = xmlnode_get_tag(si->config,"map");
   jep0033= xmlnode_new_tag("iq");
   xmlnode_put_attrib(jep0033,"to",xmlnode_get_attrib(cur, "jabber"));
 
   xmlnode_put_attrib(jep0033,"from",xmlnode_get_attrib(xmlnode_get_parent(x),"id"));
   xmlnode_put_attrib(jep0033,"type","get");
   xmlnode_put_attrib(jep0033,"id","smtp_test_jep0033");
   xmlnode_put_attrib(xmlnode_insert_tag(jep0033,"query"),"xmlns","http://jabber.org/protocol/disco#info");

   deliver(dpacket_new(xmlnode_dup(jep0033)),i);
   xmlnode_free(jep0033);


   smtp_StatsInc_(si, "Start",1);
   lcPom = (char*)pmalloc_x(i->p, 18,' ');
   smtp_GetDateTime((char *)lcPom);
   smtp_StatsSaveValue_(si, "StartLastDatetime",(char *)lcPom);

   }

smtps smtp_new(mio m)
   {
   smtps so;
   so = pmalloco(mio_pool(m), sizeof(_smtps));
   so->m = m;
   so->p = mio_pool(m);
   return so;
   }


smtpsi smtp_new_i(mio m)
   {
   smtpsi si;
   si = pmalloco(mio_pool(m), sizeof(_smtpsi));
   si->m = m;
   si->p = mio_pool(m);
   si->message = spool_new(si->p);
   return si;
   }

smtpp smtp_param(smtpi sin)
   {
   smtpp par;
   pool tmppool;

   par=pmalloco(tmppool=pool_new(), sizeof(_smtpp));
   par->tmppool=tmppool;
   par->sin=sin;

   return par;
   }
